package Devel::Chitin::OpTree::LOGOP_AUX;
use base 'Devel::Chitin::OpTree::UNOP';

our $VERSION = '0.19';

use strict;
use warnings;

1;

__END__

=pod

=head1 NAME

Devel::Chitin::OpTree::LOGOP_AUX - Deparser class for auxiliary logical OPs

=head1 DESCRIPTION

This package contains methods to deparse LOGOP_AUXs (currently none)

=head1 SEE ALSO

L<Devel::Chitin::OpTree::LOGOP>, L<Devel::Chitin::OpTree>, L<Devel::Chitin>

=head1 AUTHOR

Anthony Brummett <brummett@cpan.org>

=head1 COPYRIGHT

Copyright 2017, Anthony Brummett.  This module is free software. It may
be used, redistributed and/or modified under the same terms as Perl itself.
